package com.fusi24.rfid.ui.home;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.LinearLayoutManager;

import com.fusi24.rfid.BuildConfig;
import com.fusi24.rfid.R;
import com.fusi24.rfid.SplashScreenActivity;
import com.fusi24.rfid.adapter.StatusScanAdapter;
import com.fusi24.rfid.base.BaseActivity;
import com.fusi24.rfid.config.Constant;
import com.fusi24.rfid.data.api.RestService;
import com.fusi24.rfid.data.api.RestServiceFactory;
import com.fusi24.rfid.data.entity.DataDetail;
import com.fusi24.rfid.data.entity.DataEntryPermit;
import com.fusi24.rfid.databinding.ActivityHomeBinding;
import com.fusi24.rfid.ui.ScanActivity;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import timber.log.Timber;

public class HomeActivity extends BaseActivity implements StatusScanAdapter.OnItemSelected {

    private DataEntryPermit dataEntryPermit;
    private List<DataEntryPermit> entryPermitList;
    private List<DataDetail> statusScanList;
    private ActivityHomeBinding binding;
    private StatusScanAdapter adapter;
    private Integer idCheckType = null;
    private String idDevice;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityHomeBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);

        Toolbar toolbar = findViewById(R.id.toolbar);
        TextView toolbarTitle = findViewById(R.id.toolbar_text);

        if(toolbarTitle !=null && toolbar !=null) {
            toolbarTitle.setText(getString(R.string.app_name));
            setSupportActionBar(toolbar);
        }

        if (getSupportActionBar() != null){
            getSupportActionBar().setDisplayShowTitleEnabled(false);
        }

        initView();
        loadingData();
        getDataPermit();
        getDataStatusScan();
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.findItem(R.id.app_version)
                .setEnabled(false)
                .setTitle(BuildConfig.VERSION_NAME);
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_home, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {

        if (item.getItemId() == R.id.action_logout) {
            getSessionManager().clearData();
            startActivity(new Intent(this, SplashScreenActivity.class));
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onResume() {
        super.onResume();
        initEvent();
    }

    private void initView(){
        ActivityCompat.requestPermissions(this, Constant.PERMISSIONS_RFID, 1000);

        entryPermitList = new ArrayList<>();
        statusScanList = new ArrayList<>();

        adapter = new StatusScanAdapter(this, this);
        binding.rvStatusScan.setLayoutManager(new LinearLayoutManager(this));
        binding.rvStatusScan.setItemAnimator(new DefaultItemAnimator());
        binding.rvStatusScan.setAdapter(adapter);
    }

    @SuppressLint("SetTextI18n")
    private void loadingData(){
        idDevice = "RFID-" + Build.ID + "-" + Build.DEVICE;
        binding.tvRfidGunId.setText(idDevice);
        binding.tvRfidCardNumber.setText(getSessionManager().getUserRfidNumber());
        binding.tvEmployeeName.setText(getSessionManager().getEmployeeName());
        //TODO STILL HARDCODE
        binding.tvGateName.setText("Suzuran");
        binding.tvSiteName.setText("BMO 1");
    }

    private void initEvent(){

        binding.spinnerPermit.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                dataEntryPermit = (DataEntryPermit) parent.getItemAtPosition(position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                dataEntryPermit = (DataEntryPermit) parent.getSelectedItem();
            }
        });

        binding.btnScan.setOnClickListener(v -> {

            if (binding.spinnerPermit.getSelectedItem() == null){
                Toast.makeText(this, "Data Permit Belum Terpilih", Toast.LENGTH_SHORT).show();
                return;
            }

            if (idCheckType == null){
                Toast.makeText(this, "Status Scan Belum Terpilih", Toast.LENGTH_SHORT).show();
                return;
            }

            Intent intent = new Intent(this, ScanActivity.class);
            intent.putExtra(ScanActivity.ID_PERMIT, String.valueOf(dataEntryPermit.getId()));
            intent.putExtra(ScanActivity.ID_DEVICE, idDevice);
            intent.putExtra(ScanActivity.ID_CHECK_TYPE, idCheckType);
            startActivity(intent);
        });

    }

    private void getDataPermit() {
        RestService restService = RestServiceFactory.create(Constant.JWT_TOKEN);
        Call<List<DataEntryPermit>> call = restService.getEntryPermit();
        call.enqueue(new Callback<List<DataEntryPermit>>() {
            @Override
            public void onResponse(@NotNull Call<List<DataEntryPermit>> call, @NotNull Response<List<DataEntryPermit>> response) {
                if (response.body() != null) {
                    entryPermitList.addAll(response.body());
                    setDataPermit();
                }
            }

            @Override
            public void onFailure(@NotNull Call<List<DataEntryPermit>> call, @NotNull Throwable t) {
                Toast.makeText(HomeActivity.this, "Failed to get data permit", Toast.LENGTH_SHORT).show();
                Timber.i("DEVELOPER : %s", t.getMessage());
            }
        });
    }

    private void getDataStatusScan(){
        RestService restService = RestServiceFactory.create(Constant.JWT_TOKEN);
        Call<List<DataDetail>> call = restService.getLookUp("BeSCAN_CHECK_TYPE");
        call.enqueue(new Callback<List<DataDetail>>() {
            @Override
            public void onResponse(@NotNull Call<List<DataDetail>> call, @NotNull Response<List<DataDetail>> response) {
                if (response.body() != null){
                    statusScanList.addAll(response.body());
                    adapter.updateDetailList(statusScanList);
                    adapter.notifyDataSetChanged();
                }
            }

            @Override
            public void onFailure(@NotNull Call<List<DataDetail>> call, @NotNull Throwable t) {
                Toast.makeText(HomeActivity.this, "Failed to get data status Scan", Toast.LENGTH_SHORT).show();
                Timber.i("DEVELOPER : %s", t.getMessage());
            }
        });
    }

    private void setDataPermit(){
        ArrayAdapter<DataEntryPermit> adapter = new ArrayAdapter<>(this, android.R.layout.simple_spinner_dropdown_item, entryPermitList);
        binding.spinnerPermit.setAdapter(adapter);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        binding = null;
    }

    @Override
    public void onSelected(Integer idStatus) {
        this.idCheckType = idStatus;
    }
}
